/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: Leadtek TV2000 remote control driver.
 Valid for:

 Copyright (c) 2000 by Ivan Kolin  <ivkolin@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

//Key codes from: Vlad (DarkTrooper), <vladdt@takas.lt>

// BTTool.exe display for "Leadtek TV2000":
// GPIO_Dma_Ctl - 0000C007; (0000???? when TV window open)      ???
// GPIO_Out_En  - 000078ff;				  ------------------	???
// GPIO_Data    - remarked column (for pressed - released
//                keys and loaded/unloaded JTV).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 00ff9f04 (when JTV-window is open)    ???
// GPIO_Data    - 00ff9f02 (when JTV-window is closed)  ???


const GPIO_Dma_Ctl = 0x00c007;
const GPIO_Out_En  = 0x0078ff;
const GPIO_Data    = 0xff9f04;

KeyDef keys[]=
{
//                                         JTV is loaded        JTV is NOT loaded
//                                        Pressed    Released   Pressed   Released
// 1-st row keys layout
        // No key
        // No key
        // No key
        { 0x07, "Key_Power", "Power", },//00e21e07 - 00e21f07 ???
// 2-nd keys row
        { 0x2f, "Key_1", "1", },        //00e21e2f - 00e21f2f ???
        { 0x37, "Key_2", "2", },        //00e21e37 - ...
        { 0x3f, "Key_3", "3", },        //00e21e3f
        { 0x17, "Key_TV/FM", "", },     //00e21e17
// ...
        { 0x4f, "Key_4", "4", },        //00e21e4f
        { 0x57, "Key_5", "5", },        //00e21e57
        { 0x5f, "Key_6", "6", },        //00e21e5f
        { 0xaf, "Key_AutoScan", "", },  //00e21eaf
//...
        { 0x6f, "Key_7", "7", },        //00e21e6f
        { 0x77, "Key_8", "8", },        //00e21e77
        { 0x7f, "Key_9", "9", },        //00e21e7f
        { 0xb7, "Key_Display", "", },   //00e21eb7
//...
        { 0x8f, "Key_Recall", "Recall", },//00e21e8f
        { 0x97, "Key_0", "0", },          //00e21e97
        { 0x9f, "Key_Enter", "Enter", },  //00e21e9f
        // No key
//...
        { 0xff, "Key_CC", "", },         //00e21eff
        { 0xc7, "Key_Left", "Left", },   //00e21ec7
        { 0xcf, "Key_Right", "Right", }, //00e21ecf
        { 0xf7, "Key_Video", "", },      //00e21ef7
//...
        { 0xdf, "Key_MTS", "", },       //00e21edf
        // No key
        // No key
        { 0xa7, "Key_Mute", "", },      //00e21ea7
//...
        { 0x67, "Key_ChannelUp", "ChannelUp", },    //00e21e67

        { 0x47, "Key_VolumeDown", "VolumeDown", },  //00e21e47
        { 0x1f, "Key_FullScreen", "", },            //00e21e1f
        { 0x27, "Key_VolumeUp", "VolumeUp", },      //00e21e27

        { 0x87, "Key_ChannelDown", "ChannelDown", },//00e21e87

};

// Please, mark with ">>" sign correct answer below or add your own.
// When any key is pressed, GPIO_Data value:
// >> 1. Strongly keeps the value
//    2. In random intervals (=> ??? ms) changes to value - ????
//    3. Other case ??





typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=1000;    // default first delay for a key
INT repeat_delay=100;    // default repeat delay for a key


void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
                {
                log("> Initializing Leadtek TV2000 remote module.\n");
                log("> Copyright (c) 2000 by Ivan Kolin, <ivkolin@mail.ru>.\n");
                log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
                };



        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();

                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    INT key_pause=20; // initial repeat delay
                    DWORD test_code=0;

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | GPIO_Dma_Ctl);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | GPIO_Out_En));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | GPIO_Data);


                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;
                         DWORD key_flag=0;

                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);

                         value = (gpio_data &0x000000ff);
                         key_flag = ((gpio_data &0x00000f00)>>8);

                                 // Test block begin - comment all while build checked out driver
                                 //if (test_code != value)
                                 //{
                                 //       log(">> ---- Test information ---- \n");
                                 //       log(">>      Tick_Counter: %d\n",GetTickCount());
                                 //       log(">>         GPIO_DATA: %x\n",gpio_data);
                                 //       log(">>  Current_Key_Code: %x\n",value);
                                 //       log(">> Previous_Key_Code: %x\n",last_code);
                                 //
                                 //       log(">>    Press_Key_Flag: %x\n",key_flag);
                                 //       test_code = value;
                                 //}
                                 // Test block end

                         if (key_flag == 0xe) // any key is pressed
                         {
                             if (last_code == value)
                             {     // auto repeat for any pressed key
                                   //log(">>    Press_Key_Auto: %x\n",value); //for test
                                   AddStroke(last_code);
                                   key_pause = repeat_delay;
                             }
                             else
                             {     // first run for any pressed key
                                   //log(">>   Press_Key_First: %x\n",value); //for test
                                   last_code = value;
                                   AddStroke(last_code);
                                   key_pause = first_delay;
                             }
                         }
                         else  // no one key is pressed
                         {
                             if ((value!=last_code) && (last_code!=0xffffffff))
                             {
                                last_code = value;
                             }

                             key_pause = 20;
                         };

                    };

                log("> Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};
